require "logger"
require "logger/logging"

module TestRunner
  autoload :Assert, "test_runner/assert"
  autoload :BacktraceFilter, "test_runner/backtrace_filter"
  autoload :ColoredLogger, "test_runner/colored_logger"
  autoload :Config, "test_runner/config"
  autoload :CLI, "test_runner/cli"
  autoload :Runner, "test_runner/runner"
  autoload :Util, "test_runner/util"

  # TestRunner can be included in scripts as a mixin, e.g.
  #
  # require "test_runner"
  # include TestRunner
  #
  # logger.info "hi"
  # assert true
  #
  # You can also require "test_runner/script" to automatically include TestRunner
  def logger
    Config.logger
  end

  # Describe, context, it, and specify can help break up a larger test script
  # into chunks. Blocks are run # immediately; this is cosmetic enhancement
  # that also enables some degree of compatibility with other frameworks.
  %i(context describe it specify).each do |method_name|
    define_method method_name do |msg, &blk|
      blk.() if blk
    end
  end

  def self.included target
    Assert::Syntax.infect target
  end
end
