module TestRunner
  class ColoredLogger
    ANSI_COLORS = %i(black red green yellow blue magenta cyan white)

    DEFAULT_PALETTE = {
      :unknown => -> msg { bg(:blue, :bright, fg(:white, :bright, msg)) },
      :fatal   => -> msg { bg(:red, :bright, fg(:white, :bright, msg)) },
      :error   => -> msg { fg(:red, :bright, msg) },
      :warn    => -> msg { fg(:yellow, :bright, msg) },
      :info    => -> msg { fg(:default, :normal, msg) },
      :debug   => -> msg { fg(:cyan, :bright, msg) },
    }

    attr_reader :palette

    def initialize io, palette = DEFAULT_PALETTE
      @logger  = Logger.new io
      @palette = palette
    end

    %i(level= progname=).each do |method_name|
      define_method method_name do |*args, &block|
        @logger.public_send method_name, *args, &block
      end
    end

    %i(unknown fatal error warn info debug).each do |log_level|
      define_method log_level do |msg_or_progname = nil, &orig_block|
        colored_msg = format log_level, msg_or_progname

        if orig_block
          progname = colored_msg
          block = -> do format log_level, orig_block.() end
        else
          block = -> do colored_msg end
        end

        @logger.public_send log_level, progname, &block
      end
    end

    def format log_level, msg
      formatter = palette.fetch log_level
      instance_exec msg, &formatter
    end

    def fg *args
      col :fg, *args
    end

    def bg *args
      col :bg, *args
    end

    def col fgbg, color_code, intensity_code, str
      color_num = ANSI_COLORS.index color_code
      intensity_num = { :normal => 0, :bright => 1 }.fetch intensity_code
      fgbg_num = { :fg => 3, :bg => 4 }.fetch fgbg
      "\e[#{fgbg_num}#{color_num};#{intensity_num}m#{str}\e[0m"
    end
  end
end
