module TestRunner
  module Config
    extend self

    extend Logger::Logging

    attr_writer :child_count
    attr_writer :fail_fast
    attr_writer :internal_logger
    attr_writer :reverse_backtraces
    attr_writer :trim_backtraces

    def child_count
      Option::Number.evaluate @child_count do
        ENV.fetch "TEST_RUNNER_CHILD_COUNT", 1
      end
    end

    def fail_fast
      Option::Boolean.evaluate @fail_fast do
        ENV["TEST_RUNNER_FAIL_FAST"]
      end
    end

    def internal_logger
      if ENV["TEST_RUNNER_INTERNAL_LOGGING"]
        logger
      else
        Logger::Logging::NullLogger
      end
    end

    def reverse_backtraces
      Option::Boolean.evaluate @reverse_backtraces do
        ENV["TEST_RUNNER_REVERSE_BACKTRACES"]
      end
    end

    def trim_backtraces
      Option::Boolean.evaluate @trim_backtraces do
        ENV.fetch "TEST_RUNNER_TRIM_BACKTRACES", true
      end
    end

    class Option
      def self.evaluate value, &block
        instance = new value
        instance.fetch &block
      end

      def initialize value
        @value = value
      end

      def fetch
        @value = yield unless set?
        value
      end

      def set?
        not @value.nil?
      end

      class Number < Option
        def value
          @value.to_i
        end
      end

      class Boolean < Option
        def value
          not [nil, false, "", "0", "n", "no", "false"].include? @value
        end
      end
    end
  end
end
